package com.huyi.task.taskMan.schedule;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.huyi.task.base.entity.*;
import com.huyi.task.base.service.*;
import com.huyi.task.taskMan.service.FinanceTimedTaskService;
import com.keao.tianhu.core.util.AmtUtil;
import com.tianhu.common.core.utils.CommonUtil;
import com.tianhu.common.core.utils.DateUtils;
import com.tianhu.common.redis.common.RedisUtils;
import com.tianhu.system.api.RemoteConfigService;
import com.tianhu.system.api.RemoteSystemService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import java.text.SimpleDateFormat;
import java.util.*;

@Configuration      //1.主要用于标记配置类，兼备Component的效果。
@EnableScheduling   // 2.开启定时任务
public class FinanceTimedTask {
    private static final Logger log = LoggerFactory.getLogger(FinanceTimedTask.class);

    @Autowired
    private IZcFinanceInfService iZcFinanceInfService;
    @Autowired
    private IZcFinanceProductService iZcFinanceProductService;
    @Autowired
    private IZcFinanceRecordService zcFinanceRecordService;
    @Autowired
    private FinanceTimedTaskService financeTimedTaskService;
    @Autowired
    private RemoteSystemService remoteSystemService;
    @Autowired
    private ISysNoticeService iSysNoticeService;
    @Autowired
    private RemoteConfigService configService;
    @Autowired
    private ISysUserCompanyRelService sysUserCompanyRelService;

    /**
     * 融信消息提醒定时跑批
     */
    //每十分钟
//    @Scheduled(cron = "0 */10 * * * ?")
    //每天凌晨一点
    @Scheduled(cron = "0 0 1 * * ?")
    private void financeInformation() throws Exception{
        LambdaQueryWrapper<ZcFinanceProduct> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ZcFinanceProduct::getZfpStatus,"01");
        List<ZcFinanceProduct> productList = iZcFinanceProductService.findZcFinanceProducts(queryWrapper);
        //融信融信到期前30天提醒
        for (int i = 0; i < productList.size(); i++) {
            //最短账期
            String shortestPeriod= productList.get(i).getZfpShortestPeriod();
            Map map = new HashMap();
            //据当前时间还有30天的日期
            String day = DateUtils.getFetureDate(30+Integer.parseInt(shortestPeriod));
            map.put("day",day);
            //融信信息
            List<Map> list = financeTimedTaskService.selectFinanceInfList(map);
            if(list.size()>0){
                for(Map res :list){
                    //主键
                    String zfiId = CommonUtil.objToString(res.get("zfiId"));
                    //经办人Id
                    String user = CommonUtil.objToString(res.get("sucrUserId"));
                    //企业
                    String companyId = CommonUtil.objToString(res.get("sucrCompanyId"));
                    //融信编号
                    String zfiNumber = CommonUtil.objToString(res.get("zfiNumber"));
                    //签发有效期
                    String zfiEffectiveDate = CommonUtil.objToString(res.get("zfiEffectiveDate"));
                    //申请金额
                    String zfiAmount = CommonUtil.objToString(res.get("zfiAmount"));
                    //金额大写
                    String zfiAmountBig = AmtUtil.convert(AmtUtil.moneyFormat(zfiAmount));
                    //核心企业名称
                    String scyName = CommonUtil.objToString(res.get("scyName"));
                    //供应商名称
                    String supplierName = CommonUtil.objToString(res.get("supplierName"));
                    //供应商经办人
                    String nickName = CommonUtil.objToString(res.get("nickName"));
                    ////供应商经办人手机号
                    String phone = CommonUtil.objToString(res.get("phone"));
//                    List userList = new ArrayList();
//                    userList.add(user);
                    //到期前30天发送消息
                    //标题
                    String title = "【融信到期】【"+supplierName+"】一笔由【"+scyName+"】开立的融信还有30天到期，请联系【"+nickName+"】【"+phone+"】跟进";
                    //String title = "【融信到期】您持有的一笔融信【"+zfiAmount+"】（大写："+zfiAmountBig+"）的融信【"+zfiNumber+"】将于【"+zfiEffectiveDate+"】失效，请尽快去融资";
                    remoteSystemService.sendNotice(companyId, zfiId,title,"00","","05","0",user,"",null,null);
                }
            }
        }
        //融信融信到期前7天提醒
        for (int i = 0; i < productList.size(); i++) {
            //最短账期
            String shortestPeriod= productList.get(i).getZfpShortestPeriod();
            Map map = new HashMap();
            //据当前时间还有7天的日期
            String day = DateUtils.getFetureDate(7+Integer.parseInt(shortestPeriod));
            map.put("day",day);
            //融信信息
            List<Map> list = financeTimedTaskService.selectFinanceInfList(map);
            if(list.size()>0){
                for(Map res :list){
                    //主键
                    String zfiId = CommonUtil.objToString(res.get("zfiId"));
                    //经办人Id
                    String user = CommonUtil.objToString(res.get("sucrUserId"));
                    //企业
                    String companyId = CommonUtil.objToString(res.get("sucrCompanyId"));
                    //融信编号
                    String zfiNumber = CommonUtil.objToString(res.get("zfiNumber"));
                    //签发有效期
                    String zfiEffectiveDate = CommonUtil.objToString(res.get("zfiEffectiveDate"));
                    //申请金额
                    String zfiAmount = CommonUtil.objToString(res.get("zfiAmount"));
                    //金额大写
                    String zfiAmountBig = AmtUtil.convert(AmtUtil.moneyFormat(zfiAmount));
                    //核心企业名称
                    String scyName = CommonUtil.objToString(res.get("scyName"));
//                    List userList = new ArrayList();
//                    userList.add(user);
                    //到期前7天生成待办
                    //标题
                    String title = "【融资待办】【"+scyName+"】签发的融信【"+zfiNumber+"】将于【"+zfiEffectiveDate+"】失效，" +
                            "融信金额【"+zfiAmount+"】（大写："+zfiAmountBig+"），请尽快融资";
                    remoteSystemService.sendNotice(companyId, zfiId,title,"00","","05","0",user,"","01","/credit/goFinance/"+zfiId);
                }
            }
        }
        //融信融信到期前5天提醒
        for (int i = 0; i < productList.size(); i++) {
            //最短账期
            String shortestPeriod= productList.get(i).getZfpShortestPeriod();
            Map map = new HashMap();
            //据当前时间还有30天的日期
            String day = DateUtils.getFetureDate(5+Integer.parseInt(shortestPeriod));
            map.put("day",day);
            //融信信息
            List<Map> list = financeTimedTaskService.selectFinanceInfList(map);
            if(list.size()>0){
                for(Map res :list){
                    //主键
                    String zfiId = CommonUtil.objToString(res.get("zfiId"));
                    //经办人Id
                    String user = CommonUtil.objToString(res.get("sucrUserId"));
                    //手机号
                    String phone = CommonUtil.objToString(res.get("phone"));
                    //企业
                    String companyId = CommonUtil.objToString(res.get("sucrCompanyId"));
                    //融信编号
                    String zfiNumber = CommonUtil.objToString(res.get("zfiNumber"));
                    //签发有效期
                    String zfiEffectiveDate = CommonUtil.objToString(res.get("zfiEffectiveDate"));
                    //申请金额
                    String zfiAmount = CommonUtil.objToString(res.get("zfiAmount"));
                    //金额大写
                    String zfiAmountBig = AmtUtil.convert(AmtUtil.moneyFormat(zfiAmount));
//                    List userList = new ArrayList();
//                    userList.add(user);
                    //到期前5天发送短信
                    //标题
                    String title = "【招采云链】您持有的一笔【"+zfiAmount+"】（大写："+zfiAmountBig+"）"
                            + "的融信【"+zfiNumber+"】将于【"+zfiEffectiveDate+"】失效，请尽快处理；";
//                    remoteSystemService.sendNotice(companyId, zfiId,title,"","","","1",user,"");
                    //手机模板
                    String mould = RedisUtils.getDictValue("aliyun_message_code","zhuce");
                    //参数
                    Map map1 = new HashMap();
                    map1.put("融信编号",zfiNumber);
                    map1.put("融信金额",zfiAmount);
                    map1.put("大写",zfiAmountBig);
                    map1.put("失效期",zfiEffectiveDate);
//                    JSONObject jsonObject = new JSONObject(map1);
                    String json = JSONObject.toJSONString(map1);
                    remoteSystemService.sendAliMessage(phone,mould,json,"");
                }
            }
        }
        //融信融信到期前3天提醒
        for (int i = 0; i < productList.size(); i++) {
            //最短账期
            String shortestPeriod= productList.get(i).getZfpShortestPeriod();
            Map map = new HashMap();
            //据当前时间还有30天的日期
            String day = DateUtils.getFetureDate(3+Integer.parseInt(shortestPeriod));
            map.put("day",day);
            //融信信息
            List<Map> list = financeTimedTaskService.selectFinanceInfList(map);
            if(list.size()>0){
                for(Map res :list){
                    //主键
                    String zfiId = CommonUtil.objToString(res.get("zfiId"));
                    //融信编号
                    String zfiNumber = CommonUtil.objToString(res.get("zfiNumber"));
                    //签发有效期
                    String zfiEffectiveDate = CommonUtil.objToString(res.get("zfiEffectiveDate"));
                    //申请金额
                    String zfiAmount = CommonUtil.objToString(res.get("zfiAmount"));
                    //金额大写
                    String zfiAmountBig = AmtUtil.convert(AmtUtil.moneyFormat(zfiAmount));
                    //供应商
                    String supplierName = CommonUtil.objToString(res.get("supplierName"));
                    //平台部门Id
                    String deptId = configService.selectConfigByKey("platform_dept").getMsg();
                    //平台部门所含人员
                    String user = "";
                    LambdaQueryWrapper<SysUserCompanyRel> companyWrapper = new LambdaQueryWrapper<>();
                    companyWrapper.eq(SysUserCompanyRel::getSucrDeptId,deptId);
                    companyWrapper.eq(SysUserCompanyRel::getSucrCompanyId,"000000");
                    List<SysUserCompanyRel> userList = sysUserCompanyRelService.findSysUserCompanyRels(companyWrapper);
                    for (SysUserCompanyRel sysUser:userList) {
                        user += CommonUtil.objToString(sysUser.getSucrUserId())+",";
                    }
//                    List userList = new ArrayList();
//                    userList.add(user);
                    //到期前3天生成平台内部待办
                    //标题
                    String title = "【融资待办】【"+supplierName+"】融信【"+zfiNumber+"】融信金额【"+zfiAmount+"】（大写："+zfiAmountBig+"）" +
                            "将于【"+zfiEffectiveDate+"】失效，请相关人员跟进！";
                    remoteSystemService.sendNotice("000000", zfiId,title,"00","","07","0",user,"","00",null);
                }
            }
        }


    }

    /**
     * 融信失效定时跑批
     */
    //每十分钟
//    @Scheduled(cron = "0 */10 * * * ?")
    //每天凌晨一点
    @Scheduled(cron = "0 0 1 * * ?")
    private void financeStatus() throws Exception{
        //融信到期 => 融信失效,待办删除
        Map map = new HashMap();
        List<Map> list = financeTimedTaskService.selectFinanceInfDetail(map);
        for (int i = 0; i < list.size(); i++) {
            //融信id
            String zfiId = CommonUtil.objToString(list.get(i).get("zfiId"));
            //签发有效期
            String zfiEffectiveDate = CommonUtil.objToString(list.get(i).get("zfiEffectiveDate"));
            //最短账期
            int zfpShortestPeriod = Integer.parseInt(list.get(i).get("zfpShortestPeriod")+"");
            //String 转 Date
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");//注意月份是MM
            Date date = simpleDateFormat.parse(zfiEffectiveDate);
            //签发有效期 - 最短账期(天数) = 失效日期
            GregorianCalendar cal = new GregorianCalendar();
            cal.setTime(date);
            cal.add(Calendar.DATE, -zfpShortestPeriod);
            //失效日期
            String expirationDate=DateUtils.parseDateToStr("yyyy-MM-dd",cal.getTime());
            //获取当前时间
            String currentDate = DateUtils.getDate();
            //失效日期 = 获取当前时间 修改融信状态,删除待办
            if (expirationDate.equals(currentDate)){
                //修改融信表
                ZcFinanceInf zcFinanceInf = new ZcFinanceInf();
                //主键
                zcFinanceInf.setZfiId(zfiId);
                //状态(00:待确权 01:待签收 02:已生效 03:已失效 04:融资中 05:已融资 06:平台退回 07:已失效(到期失效) 99:删除)
                zcFinanceInf.setZfiStatus("07");
                //修改待办表
                LambdaQueryWrapper<SysNotice> queryWrapper = new LambdaQueryWrapper<>();
                queryWrapper.eq(SysNotice::getTableId,zfiId);
                SysNotice sysNotice = new SysNotice();
                sysNotice.setStatus("2");
                iSysNoticeService.update(sysNotice,queryWrapper);
            }

        }
    }

    /**
     * 融资还款逾期定时跑批
     */
    //每十分钟
//    @Scheduled(cron = "0 */10 * * * ?")
    //每天凌晨一点
    @Scheduled(cron = "0 0 1 * * ?")
    private void recordStatus() throws Exception{
        Map map = new HashMap();
        List<Map> list = financeTimedTaskService.selectFinanceInfRecord(map);
        for (int i = 0; i < list.size(); i++) {
            //融资纪录id
            String zfrId = CommonUtil.objToString(list.get(i).get("zfrId"));
            //承诺还款时间
            String zfrRepaymentDate = CommonUtil.objToString(list.get(i).get("zfrRepaymentDate"));
            //String 转 Date
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");//注意月份是MM
            Date date = simpleDateFormat.parse(zfrRepaymentDate);
            //取得指定时间间隔后的系统时间
            GregorianCalendar calendar = (GregorianCalendar) Calendar.getInstance();
            calendar.add( Calendar.DAY_OF_MONTH, 0);
            System.out.println("比较时间"+simpleDateFormat.format(calendar.getTime()));
            System.out.println("还款时间"+simpleDateFormat.format(date));
            if(simpleDateFormat.format(calendar.getTime()).compareTo(simpleDateFormat.format(date))>0){
               ZcFinanceRecord zcFinanceRecord = new ZcFinanceRecord();
               zcFinanceRecord.setZfrId(zfrId);
               //还款状态(00:待还款, 01: 已还款, 02: 已逾期)
               zcFinanceRecord.setZfrApplyStatus("02");
               zcFinanceRecordService.updateZcFinanceRecord(zcFinanceRecord);
            }
        }
    }
}

