package com.huyi.task.taskMan.schedule;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.huyi.task.base.entity.PubFileInf;
import com.huyi.task.base.entity.SysDictData;
import com.huyi.task.base.service.IPubFileInfService;
import com.huyi.task.base.service.ISysDictDataService;
import com.huyi.task.taskMan.mapper.FtpFileTaskMapper;
import com.huyi.task.utils.FtpClientUtil;
import lombok.SneakyThrows;
import org.apache.commons.net.ftp.FTPClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import java.util.*;

@Configuration      //1.主要用于标记配置类，兼备Component的效果。
@EnableScheduling   // 2.开启定时任务
public class FtpFileTask {
    @Autowired
    private FtpFileTaskMapper fileTaskMapper;
    @Autowired
    private ISysDictDataService dictDataService;
    @Autowired
    private IPubFileInfService fileInfService;

    @SneakyThrows
    @Scheduled(cron = "0 0 1 * * ?")
    private void clearFile(){
        // 获取当前日期的前5天
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(new Date());
        calendar.add(Calendar.DATE, -5);
        Date beforeDate = calendar.getTime();

        // 查询需要删除的文件
        Map<String, Object> param = new HashMap<>();
        param.put("beforeDate", beforeDate);
        List<Map<String, String>> fileList = fileTaskMapper.queryDeletedFile(param);

        LambdaQueryWrapper<SysDictData> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(SysDictData::getDictType, "pub_doc_config");
        List<SysDictData> dictDataList = dictDataService.findSysDictDatas(lambdaQueryWrapper);
        String ftp_ip = null,ftp_username = null,ftp_password = null;
        int ftpPort = 21;
        for (SysDictData dictData : dictDataList) {
            if ("01".equals(dictData.getDictValue())) {
                ftp_username = dictData.getDictLabel();
                continue;
            } else if ("02".equals(dictData.getDictValue())) {
                ftp_password = dictData.getDictLabel();
                continue;
            } else if ("03".equals(dictData.getDictValue())) {
                ftp_ip = dictData.getDictLabel();
                continue;
            } else if ("04".equals(dictData.getDictValue())) {
                ftpPort = Integer.parseInt(dictData.getDictLabel());
                continue;
            }
        }
        // 登陆
        FTPClient ftp = FtpClientUtil.getConnectionFTP(ftp_ip, ftpPort, ftp_username, ftp_password);
        for (Map<String, String> fileMap : fileList) {
            String fileUrl = fileMap.get("fileUrl");
            String fileName = fileUrl.split("\\/")[fileUrl.split("\\/").length-1];
            boolean success = FtpClientUtil.deleteFile(ftp, fileMap.get("filePath"), fileName);
            if (success) {
                PubFileInf fileInf = new PubFileInf();
                fileInf.setPfiFileId(fileMap.get("fileId"));
                fileInf.setPfiIsDel("99");
                fileInf.setPfiIsDeal("1");
                fileInfService.updatePubFileInf(fileInf);
            }
        }
        // 退出登录
        ftp.logout();
    }
}
